# -*- coding: utf-8 -*-

import os
import json
import logging
from datetime import datetime, timedelta

import pandas as pd

from ydl_ai_recommender.src.data.mysql_client import MySQLClient
from ydl_ai_recommender.src.utils import get_conf_path, get_data_path


class DBDataManager():
    """
    导出/更新数据库中的数据到本地文件
    """

    def __init__(self) -> None:
        self.local_file_dir = get_data_path()
        self.logger = logging.getLogger(__name__)
        self.client = MySQLClient.create_from_config_file(get_conf_path())


    def _load_local_plain_file(self, name):
        with open(os.path.join(self.local_file_dir, name), 'r', encoding='utf-8') as f:
            return f.readlines()


    def _load_local_json_file(self, name):
        with open(os.path.join(self.local_file_dir, name), 'r', encoding='utf-8') as f:
            return json.load(f)


    def _save_plain_data(self, lines, name, mode='w'):
        self.logger.info('开始保存 %s 到本地', name)
        with open(os.path.join(self.local_file_dir, name), mode, encoding='utf-8') as f:
            f.write('\n'.join(lines))
        self.logger.info('%s 保存成功，共保存 %s 行数据', name, len(lines))


    def _save_json_data(self, data, name):
        self.logger.info('开始保存 %s 到本地', name)
        with open(os.path.join(self.local_file_dir, name), 'w', encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False)
        self.logger.info('%s 保存成功，共保存 %s 个对象', name, len(data))


    def update_profile(self):
        # TODO 用户画像字段
        select_fields = ['*']
        sql = 'SELECT {} FROM ads.ads_register_user_profiles'.format(', '.join(select_fields))
        sql += ' WHERE uid IN (SELECT DISTINCT uid FROM ods.ods_ydl_standard_order)'
        _, all_data = self.client.query(sql)
        df = pd.DataFrame(all_data)
        df.to_excel(os.path.join(self.local_file_dir, 'all_profile.xlsx'), index=None)
        # self._save_json_data(all_data, 'all_profile.json')


    def update_order_info(self):
        select_fields = ['main_order_id', 'uid', 'supplier_id', 'price', 'standard_order_type']
        select_fields.append('DATE_FORMAT(update_time, "%Y-%m-%d") AS update_time')
        sql = 'SELECT {} FROM ods.ods_ydl_standard_order'.format(', '.join(select_fields))
        _, all_data = self.client.query(sql)
        df = pd.DataFrame(all_data)
        df.to_excel(os.path.join(self.local_file_dir, 'all_order_info.xlsx'), index=None)


    def _load_order_data(self, conditions=None):
        select_fields = ['main_order_id', 'uid', 'supplier_id', 'price', 'standard_order_type']
        select_fields.append('DATE_FORMAT(update_time, "%Y-%m-%d") AS update_time')

        sql = 'SELECT {} FROM ods.ods_ydl_standard_order'.format(', '.join(select_fields))
        if conditions:
            sql += ' WHERE {}'.format('AND '.join(conditions))
        
        self.logger.info('开始执行sql %s', sql)
        cnt, data = self.client.query(sql)
        self.logger.info('sql执行成功，共获取 %s 条数据', cnt)
        return data


    def load_test_data(self, days=5):
        now = datetime.now()
        start_time = now - timedelta(days=days)
        conditions = [
            'create_time >= "{}"'.format(start_time.strftime('%Y-%m-%d')),
        ]
        order_data = self._load_order_data(conditions=conditions)
        df = pd.DataFrame(order_data)
        df.to_excel(os.path.join(self.local_file_dir, 'test_order_info.xlsx'), index=None)

        select_fields = ['*']
        sql = 'SELECT {} FROM ads.ads_register_user_profiles'.format(', '.join(select_fields))
        sql += ' WHERE uid IN (SELECT DISTINCT uid FROM ods.ods_ydl_standard_order WHERE create_time >= "{}")'.format(start_time.strftime('%Y-%m-%d'))
        _, all_data = self.client.query(sql)
        df = pd.DataFrame(all_data)
        df.to_excel(os.path.join(self.local_file_dir, 'test_profile.xlsx'), index=None)


if __name__ == '__main__':
    manager = DBDataManager()
    manager.load_test_data()
    # manager.update_local_data()
    # print(manager.make_index())